#!/usr/bin/env python
#coding:utf-8
# Author:   --<>
# Purpose: Create junctions and index for a transcriptome analysis
# ver.4  - only unique junctions are written <-- old!
# ver 0.2 - adjust by sequence length
# Latest update: 090810
# 0.3 removed old create_junction function and renamed new one
# Created: 8/29/2011
# Usage:
# python createJunctions.py -c cfg file

from optparse import OptionParser
import logging, os, sys, time
#from configparser import ConfigParser, RawConfigParser
import utils

__doc__ = "createJunctions.py"

t0 = time.time()


# Place below in a configuration file or command line option?
#------------------------------------------------------------------------------------------
trim_for_junction = 2
pad = "NNNNNN"

# specific aligner options? BWA parametres?
#--------------------------------------------------------------------------------------------

idx=[]


class Gene(object):   
    def __init__(self):
        self.__class__.__name__
        self.id=''  # ensemble id 
        self.start=-1
        self.end=-1
        self.strand =''
        self.description=''
        self.chr = ''
        self.transcripts={} # transcript id : transcript object
                
        
class Transcript(object):
    def __init__(self):
        self.__class__.__name__
        self.id=''
        #self.start=''
        self.strand=''
        self.exon_meta={} # exon rank in this transcript : exon id
        

class Exon(object):
    def __init__(self):
        self.__class__.__name__
        self.id=''
        self.start=-1
        self.end=-1
        self.sequence=''
        self.chr=''
        self.length=-1


# store junction objects during dup seq removal step
class Junction(object):
    def __init__(self):
        self.__class__.__name__
        self.start=''
        self.end=''
        self.coords=[]
        self.strand=''
        


def getRefGenome(r):
    # read entire reference fasta sequence into memory
    # create sequence object for each chromosome?
    # if this is too slow, perhaps better to read separate chromosome files?
    # This seems to work quite fast 
    chrSeq = {}
    chrOrder = []

    x = open(r,'r')
    seq = ''
    c=''
    i=0
    for line in x:
        
        if ">" in line:
            
            if i>0:
                chrSeq[c]=seq
                                
            i+=1
            # get only chromosome names 
            c = line.split()[0][1:]
            
            # hardcoded to remove 'chr' from chromosome names
            if c[0:3]=='chr':
                c = c[3:]
            
            ### Add chromosome to chrOrder
            chrOrder.append(c)
        
            # reset seq to store the next chromosome sequence 
            seq=''
                
            continue
        # remove the new line character at the end of each sequence line
        seq+=line[:-1].upper()
        
    chrSeq[c]=seq
    
    
    return chrSeq,chrOrder


def get_exon_information(transFile, chrSeq, full_outdir):
    """
     transFile is a delimited list of exon start and end points
      by transcript name, chromosome, exon start/stop and rank.
    This can be a custom list or if using Ensembl, download a file
     with the following headers:
     
     Ensembl Transcript ID   
     Chromosome Name 
     Exon Chr Start (bp)     
     Exon Chr End (bp)       
     Exon Rank in Transcript
     (NOTE: aways remove all header before using this file)
     
     Using the appropriate genome sequence, and exon fasta file with
     full exon sequence and a junctions fasta file and index is created
     for repositioning.
    """
    # get exon information from given coordinates
    # log.info(exon sequences fasta for use by converJunctions
    
    trans = open(transFile,'r')
    
    exonFaFile = os.path.join(full_outdir,"exons.fa")  # need to make standard reference name
    exonsFasta = open(exonFaFile,'w')
    
    nonMatchingChr=[]
    skippedExonCount = 0
    
    exons={}

    e=''
    chrCurrent=''
    
    chrTranscripts={}
    
    for chromosome in chrSeq:
        chrTranscripts[chromosome]={}
        
    count=0
    for line in trans:
        
        # expect the first line to be a header
        if count==0:
            count+=1
            continue
        
        i = line.split()
    
        transcript_id = i[0]
        chromosome = i[1]
        exon_start = int(i[2])  #1-based inclusive format
        exon_end = int(i[3])    #1-based inclusive format
        exon_rank = i[4]
        strand = i[5]
        
        if strand == "+":
            strand = "1"
        if strand == "-":
            strand = "-1"
            
        if chromosome in chrSeq:
            chrCurrent=chromosome
        else:
            # chromosome is not in matching genome fasta file
            skippedExonCount += 1
            nonMatchingChr.append(chromosome)
            continue

        e = Exon()
        e.chr = chromosome
        e.start = exon_start
        e.end = exon_end
        e.rank = exon_rank
        e.length = int(e.end)-int(e.start)+1
            
        seq = chrSeq[chromosome]
        e.sequence=seq[int(e.start)-1:int(e.end)]
        
        transcripts = chrTranscripts[chromosome]
        
        t=''
        if transcript_id in transcripts:
            t = transcripts[transcript_id]
        else:
            t = Transcript()
            t.id = transcript_id
            
        t.exon_meta[exon_rank]=e
        t.strand = strand
        transcripts[transcript_id]=t
        
        # update the chrTranscripts record for each chromosome
        chrTranscripts[chromosome]=transcripts
            
        # create fasta file of exons
        transcriptName = ">"+transcript_id
        header = transcriptName,e.chr,str(e.start),str(e.end),e.rank,strand
        header = "|".join(header)
        exonsFasta.write(header)
        exonsFasta.write('\n')
        exonsFasta.write(e.sequence)
        exonsFasta.write('\n')
        count+=1
        
    # log non-matching chromosomes from transcript model
    nonMatchingChr = set(nonMatchingChr)
    nmctxt = os.path.join(full_outdir, "nonMatchingChr.txt")
    nm = open(nmctxt,'w')
    for i in nonMatchingChr:
        log.info("Chromosome " + i + " is not included in the genomic fasta")
        nm.write(i)
        nm.write('\n')
        
    log.info("Total exons:" + str(count))
    log.info("Total skipped exons:" + str(skippedExonCount))
    
    return chrTranscripts


def revcomp(seq):
    # reverse complement sequence (in captitals)
    # i.e. convert: 'TAATTCCCAAGTTCCTTATGGCCAATGGTCAGTTGGTGAAGATGCTGCTT'
    #           to: 'AAGCAGCATCTTCACCAACTGACCATTGGCCATAAGGAACTTGGGAATTA'
    trans={'A':'T','T':'A','G':'C','C':'G','N':'N'}
    j=len(seq)-1
    revcompseq=trans[seq[j:]]
    i=0
    while j > 0:
        j-=1
        i-=1
        revcompseq+=trans[seq[j:i]]
    return revcompseq
    

def parse_transcript_file(transcript_file):
    log.info("=> reading transcript file...")
    file = open(transcript_file,'r')
    i=0
    transcripts={}
    
    for line in file:
        i = line.split()
        
        transcript_id = i[0]
        chromosome = i[1]
        exon_start = i[2]
        exon_end = i[3]
        exon_rank = i[4]
        strand = i[5]
        
        exon_id = transcript_id + "_" + exon_rank
        
        if transcript_id in transcripts:
            t = transcripts[transcript_id]
        else:
            t = Transcript()
            t.id = transcript_id
            
        if exon_rank in t.exon_meta:
            log.info("Error => an exon with this rank already exists!")
        else:
            t.exon_meta[exon_rank]=exon_id
            
        transcripts[transcript_id]=t
    return transcripts


def print_num_exons(g):
    for i in g:
        #log.info(g[i].exons
        gene = g[i]
        log.info(gene.id + gene.numExons())
        

def setJunctionRank(exons, sl):
    """ 
     Given exon rank and size, calculate the exons whose sequence 
     will be part of a given junction reference sequence
     Implemented to address reads that span exons
    """
   #exons={1:104, 2:82, 3:35, 4:113, 5:46, 6:187, 7:94}
    #log.info(exons
    #sl = int(config.get('RUN_TIME', 'seq_length'))
    nextExon=0
    i=1
    junctionRank=[]
    while i < len(exons):
        #skip the last exon
        if i+1> len(exons):
            break
        
        exonLength = exons[i]
        
        if exons[i+1] >= sl:
            # for both exons, read is smaller -- most common
            ###junction = [exons[i],exons[i+1]]
            exonSet = [i,i+1]
            i+=1
            
        elif exons[i+1] < sl and i<=len(exons):
            ###junction = [exons[i]]
            exonSet = [i]
            i+=1

            k=1
            while k < 11:
                #Append exon to exonSet untill the loop hits an exon (exonN) with length >=sl --> exit loop --> append exonN to exonSet --> exit the elif loop
                if i<=len(exons):
                    if exons[i] < sl:
                        ###junction.append(exons[i])
                        exonSet.append(i)
                        i+=1
                k+=1
                
            if i<=len(exons):
                ###junction.append(exons[i])
                exonSet.append(i)
                
        
        #log.info(junction
        #log.info(exonSet
        junctionRank.append(exonSet)
    #log.info("junctionRank:",junctionRank
    return junctionRank

        
def modifyExonSizeMeta(exon_meta, seq_length):
    exonRankSize = {}
    for rank in exon_meta:
        e = exon_meta[rank]
        exonRankSize[int(rank)]=e.length
        
    junctionRank = setJunctionRank(exonRankSize, seq_length)
    
    return junctionRank
        

def progressUpdate(geneCount,timeCount,noGenes):
    percentComplete = float(geneCount)/noGenes*100
    percentComplete = "%2.0f" % percentComplete
    percentComplete += "%"
    minutesElapsed = (time.time()- t0)/60
    minutesElapsed = "%2.0f" % minutesElapsed
    log.info("Genes processed:"+geneCount+"/"+noGenes+" ("+percentComplete+") "+minutesElapsed+" minutes elapsed.")
 
 
def create_junctions(chromosome, chrTranscripts, chrSeq, seq_length):
    #log.info("=> creating junction sequence and index files by chromosome:")
    log.info("--> " + chromosome)
    timeCount=1000

    junctionCoordCount={}
    junctionArray=[]
    
    exonJunctionLength = seq_length+trim_for_junction #trim_for_junction=2
    
    #for c in chrSeq:
        # c contains an individual chromosome name
        ###JNFilename = os.path.join(full_outdir,c+"_JN")
        ###JNFile = open(JNFilename,'w')
        
        #idxFilename = fastaFilename+".idx"
                
    count=1
    junctionCoordCount[chromosome]=count
    
    #for chromosome in chrTranscripts:
    #log.info("--> " + chromosome)
    transcripts = chrTranscripts[chromosome]
    
    for transcript_id in transcripts:
            
        t = transcripts[transcript_id]
        strand = t.strand
        exon_meta = t.exon_meta
        numExons = len(exon_meta)
        
        junctionRank = modifyExonSizeMeta(exon_meta, seq_length)
        ###log.info("transcript_id:" + transcript_id + ",junctionRank:" + junctionRank)###debugging
        for junction in junctionRank:
            noJunctionExons = len(junction)
            k=0
                
            e1=Exon()
            e2=Exon()
            exonList=[]
                
            coord=[]
            #log.info(exon_meta
            for exonRank in junction:
                exonRank = str(exonRank)
                #exon_id = exon_meta[str(exonRank)]
                #exonList.append(exon_id)
                if k==0:
                    # first exon in junction set
                    e1 = exon_meta[exonRank]
                        
                    #seq1 = e1.sequence[-seq_length+trim_for_junction:]
                    chromosome = e1.chr
                    
                    if chromosome not in chrSeq:
                        continue
                        
                    # get coord for junction sequences
                    if e1.length >= exonJunctionLength:
                        if strand == '-1':
                            coord.append(str(e1.start)) # the actual end of the exon
                            coord.append(str(e1.start+seq_length-trim_for_junction-1))
                            #coord.append(str(e1.start+seq_length-trim_for_junction))
                        else:
                            coord.append(str(e1.end-seq_length+trim_for_junction+1))
                            #coord.append(str(e1.start+seq_length-trim_for_junction))
                            coord.append(str(e1.end))
                    else:
                        coord.append(str(e1.start))
                        coord.append(str(e1.end))
                                
                elif k==noJunctionExons-1:
                    # last exon in junction set
                    e2 = exon_meta[exonRank]
                    #seq2 = e2.sequence[:seq_length-trim_for_junction]        
                    
                    # get coord
                    if e2.length >= exonJunctionLength:
                        if strand == '-1':
                            coord.append(str(e2.end-seq_length+trim_for_junction+1))
                            coord.append(str(e2.end))
                        else:
                            coord.append(str(e2.start))
                            coord.append(str(e2.start+seq_length-trim_for_junction-1))
                    else:
                        coord.append(str(e2.start))
                        coord.append(str(e2.end))
                        
                else:
                    # any exons < read_length which will lie in middle of junction sequence
                    e3 = exon_meta[exonRank]
                    #seq3+=e3.sequence
                    coord.append(str(e3.start))
                    coord.append(str(e3.end))
                        
                k+=1
                    

            if chromosome not in chrSeq:
                continue
                                           
            ### Get exon-exon junction sequence from the reference genome
            # sort coordinates that will be used for exon-exon junctions and 
            # build the sequence from the supplied reference genome
            """
            genomeSeq = chrSeq[chromosome]
            seqFromGenome = ''
            """
            coordSorted=[]
            for k in coord:
                coordSorted.append(int(k))
            coordSorted.sort()
            """
            m=0
            while m < len(coordSorted):
                seqFromGenome += genomeSeq[coordSorted[m]-1:coordSorted[m+1]]
                m+=2
                
                    
            ### Make sure junction sequences in output are unique        
            # checkf for 100% match; should this be modified to calculate a percentage?
            junction_seq = junctionSeq[chromosome]
            if seqFromGenome in junction_seq:
                # log.info("sequence already in database"
                # just add information on the other transcripts
                trans_id = junction_seq[seqFromGenome]
                trans_id.append(t)
                junction_seq[seqFromGenome]=trans_id 
                
            else:
                junction_seq[seqFromGenome]=[t]
            """
                
            ###JNFilename = os.path.join(full_outdir,chromosome+"_JN.header")
                ###junctionsFasta = open(JNFilename,'a')
            """junctionSeqForReferenceFilename = os.path.join(full_outdir,chromosome+"_JN.txt")
            junctionSeqForReference = open(junctionSeqForReferenceFilename,'a')"""
    
            """seqpad = seqFromGenome+pad+'\n'

            fastaSeqLength = len(seqFromGenome+pad)
            currentFastaRefLength = junctionCoordCount[chromosome]
            junctionCoordCount[chromosome]+=fastaSeqLength
            countInfo = str(currentFastaRefLength)+"-"+str(currentFastaRefLength+fastaSeqLength-1)
            """
            coord=[]
            for k in coordSorted:
                coord.append(str(k))
            
            coordList = ','.join(coord)
            #header = ">"+ coordList +"|"+countInfo+"|"+strand
            ###header = ">" + coordList + "|" +strand
            header = coordList + "|" +strand
            junctionArray.append(header)
            ###junctionsFasta.write(header)
            ###junctionsFasta.write("\n")
            """
            junctionsFasta.write(seqpad)
            junctionSeqForReference.write(seqpad)
            junctionSeq[chromosome]=junction_seq
            """
    #print("junctionArray "+str(len(junctionArray)))
    return junctionArray
   


# trim the starting sequence of a duplicated junction sequence region 
def trim_start_dup(read_length,j_object,j):
    subject_coords = list(j_object.coords)
    coords = j.coords
    log.debug("enter trimming...")
    n=0
    pos=''
    pos1='' # to record the last poped position from the current coords
    pos2=''
    while n<len(subject_coords)-3:
     
        log.debug("n=%d" %n)
        if int(coords[0])==int(subject_coords[n]) and len(coords)>4:
            # remove the entire pair
            pos1 = coords.pop(0)
            pos2 = coords.pop(0)
            # update pos to confirm removal if both pos1 and pos2 match the existing sequence 
            if pos2==subject_coords[n+1]:
                pos = pos2
            # if not both pos1 and pos2 match, then put them back and leave the loop without updating pos
            else:
                coords.insert(0,pos2)
                coords.insert(0,pos1)
                break
            n+=2
        else:
            break
    
    if n==0:
        return coords
    else:
        if j.strand=="1":
            short_exon_length = int(coords[1])-int(coords[0])+1
            if short_exon_length<(int(read_length)-trim_for_junction):
                new_start = int(pos)-(int(read_length)-trim_for_junction-short_exon_length)+1 #strand==1
                coords.insert(0,pos)
                coords.insert(0,str(new_start))
                log.debug("new start exon %s %d" %(pos, new_start))

        else: 
            #coords has been reversed so the front pos > later positions
            short_exon_length = int(coords[0])-int(coords[1])+1
            if short_exon_length<(int(read_length)-trim_for_junction):
                new_start = int(pos)+(int(read_length)-trim_for_junction-short_exon_length)-1
                coords.insert(0,pos)
                coords.insert(0,str(new_start))
                log.debug("new start exon %s %d" %(pos, new_start))

    return coords
        

def trim_end_dup(read_length,j_object,j):
    subject_coords = list(j_object.coords)
    log.debug("j_object coords before trim %s" %j_object.coords)
    coords = j.coords
    # flip the order so the end is coords[0]
    subject_coords.reverse()
    coords.reverse()
    log.debug("enter end trimming...")
    n=0
    pos=''
    pos1='' # to record the last poped position from the current coords
    pos2=''
    while n<len(subject_coords)-3:
        log.debug("n=%d" %n)
        if int(coords[0])==int(subject_coords[n]) and len(coords)>4:
            # remove the entire pair
            pos1 = coords.pop(0)
            pos2 = coords.pop(0)
            if pos2==subject_coords[n+1]:
                pos = pos2
            else:
                log.debug("add pos1 and pos2 back %s %s" %(pos2, subject_coords[n+1]))
                coords.insert(0,pos2)
                coords.insert(0,pos1)
                log.debug("leave while loop 1")
                log.debug("%s" %coords)
                log.debug("pos=%s" %pos)
                break
            log.debug(coords)
            log.debug("pos=%s" %pos)
            n+=2
        else:
            log.debug("leave while loop")
            break
    
    if n==0:
        coords.reverse()
        return coords
    else:
        if j.strand=="1":
            log.debug("coords[0],coords[1], %s, %s" %(coords[0],coords[1]))
            #print("j.coords[n-1], j.coords[n-2]", len(j.coords))
            short_exon_length = int(coords[0])-int(coords[1])+1
            if short_exon_length<(int(read_length)-trim_for_junction):
                new_end = int(pos)+(int(read_length)-trim_for_junction-short_exon_length)-1
                coords.insert(0,pos)
                coords.insert(0,str(new_end))
                log.debug("new end exon 1 %s %d" %(pos, new_end))
        else: 
            #coords has been reversed so the front pos > later positions
            log.debug("coords[0],coords[1], %s, %s" %(coords[0],coords[1]))
            short_exon_length = int(coords[1])-int(coords[0])+1
            if short_exon_length<(int(read_length)-trim_for_junction):
                new_end = int(pos)-(int(read_length)-trim_for_junction-short_exon_length)+1
                coords.insert(0,pos)
                coords.insert(0,str(new_end))
                log.debug("new end exon -1 %s %d" %(pos, new_end))
            
    coords.reverse()
    return coords

#>1022955,1022977,1025733,1025808,1026852,1026945,1027371,1027468|1077453-1077749|-1
def remove_start_dup(read_length,junction_collection,j): 
    
    jun_start = j.start
    coords = j.coords
    strand = j.strand
    old_coord_str = ','.join(coords)
    # SANITY CHECK
    if len(coords)<4:
        log.info("wrong length of junction sequence %s" %old_coord_str)
        sys.exit(-1)
    for j_object in junction_collection[jun_start]:
        # compare transcripts with the same start pos and orentation
        if j_object.coords==coords:
            log.debug("same sequence %s %s" %(j_object.coords, coords))
            j.start = "0"
            return j
        else:
            start_pair = [coords[0],coords[1]]
            j_object_start_pair = [j_object.coords[0],j_object.coords[1]]
            log.debug("existing coords %s" %j_object.coords)
            if start_pair == j_object_start_pair:
                if len(coords)==4 and len(j_object.coords)>4:
                    log.debug("skip 1")
                    continue
                elif len(j_object.coords)==4 and len(coords)>4:
                    log.debug("skip 2")
                    continue
                else:
                    coords = trim_start_dup(read_length,j_object,j)
                    if jun_start == coords[0]:
                        log.debug("Start was not trimmed") 
                        continue
                    else:
                        jun_start = coords[0]
                        j.start = jun_start
                        j.coords = coords
                        new_coord_str = ','.join(coords)
                        log.debug("junction seq %s is trimmed to %s" %(old_coord_str,new_coord_str))
                        return j
    return j


def remove_end_dup(read_length,junction_collection,j): 
    jun_end = j.end
    coords = j.coords
    strand = j.strand
    old_coord_str = ','.join(coords)
    # SANITY CHECK
    if len(coords)<4:
        log.info("wrong length of junction sequence %s" %old_coord_str)
        sys.exit(-1)
    for j_object in junction_collection[jun_end]:
        # compare transcripts with the same start pos and orentation
        if j_object.coords==coords:
            #regardless of strand orientation
            log.debug("same sequence %s %s"%(j_object.coords,coords))
            j.end = "0"
            return j
        else:
            end_pair = [coords[-1],coords[-2]]
            j_object_end_pair = [j_object.coords[-1],j_object.coords[-2]]
            if end_pair == j_object_end_pair:
                if len(coords)==4 and len(j_object.coords)>4:
                    continue
                elif len(j_object.coords)==4 and len(coords)>4:
                    continue
                else:
                    log.debug("existing coords  %s" %j_object.coords)
                    coords = trim_end_dup(read_length,j_object,j)
                    log.debug("j_object coords after trim end %s" %j_object.coords)
                    if jun_end == coords[-1]:
                        log.debug("End was not trimmed") 
                        #the previous records possiblly contain 4 coords or this is the shortest possible trimmed sequence")
                        continue
                    else:
                        jun_end = coords[-1]
                        j.end = jun_end
                        j.coords = coords
                        #seq = seq[(int(jun_start)-int(last_coords[0])):]
                        new_coord_str = ','.join(coords)
                        log.debug("junction seq %s is trimmed to %s" %(old_coord_str,new_coord_str))
                        return j
    return j


# modify the dup junction sequences in the chromosome
def modify_junction_sequences(read_length, junctionArray):
    ###log.info("=> reading junction_fa...")
    ###file = open(junction_fa,'r')
    # to collect all junction objects
    junction_start_collection={}
    junction_end_collection={}
    junction_collection=[] #junction_collection={'start_coord':[j1,j2,j3,...,jn]} #junction_collection={'start_coord':[j1,j2,j3,...,jn]}
    i=0
    #print("junctionArray "+str(len(junctionArray)))
    #for line in file:
    for line in junctionArray:
        # process header lines
        ###if ">" in line:
        i+=1
        # get the junction start coord
        ###(coord_string,strand) = line[:-1].split("|")
        ###coords = coord_string[1:].split(",")
        (coord_string,strand) = line.split("|")
        coords = coord_string.split(",")
        # coords are always sorted from the actual start position to the actual end position
        if strand=="-1":
            coords.reverse()
        jun_start = coords[0]
        jun_end = coords[-1]
        
        j = Junction()
        j.start = jun_start
        j.end = jun_end
        j.coords = coords
        j.strand = strand
        while jun_start in junction_start_collection:
            j = remove_start_dup(read_length,junction_start_collection,j)
            if j.start == "0":
                log.debug("Remove dup sequence %s" %coord_string[1:])
                break
            elif j.start == jun_start:
                log.debug("the previous records possiblly contain 4 coords or this is the shortest possible trimmed sequence")
                break
            else:
                jun_start = j.start
            
        #if jun_start==0, it means there's a same sequence in the junction_collection already with the same coords
        if j.start == "0":
            pass
        else:
            #if jun_end in junction_end_collection:
            while jun_end in junction_end_collection:
                j = remove_end_dup(read_length,junction_end_collection,j)
                if j.end == "0":
                    log.debug("Remove dup sequence %s" %coord_string[1:])
                    break
                elif j.end == jun_end:
                    log.debug("the previous records possiblly contain 4 coords or this is the shortest possible trimmed sequence")
                    break
                else:
                    jun_end = j.end
                
            if j.end == "0":
                pass
            else:
                junction_collection.append(j)
                if j.start in junction_start_collection:
                    log.debug("add start entry %s" %coords)
                    junction_start_collection[j.start].append(j)
                else:
                    log.debug("new start entry 1 %s" %coords)
                    junction_start_collection[j.start]=[j]
                if j.end in junction_end_collection:
                    log.debug("add end entry %s" %coords)
                    junction_end_collection[j.end].append(j)
                else:
                    log.debug("new end entry 1 %s" %coords)
                    junction_end_collection[j.end]=[j]
    log.debug(len(junction_collection))
    log.debug(len(junction_start_collection))
    log.debug(len(junction_end_collection))
    log.debug("%d JUNCTION SEQUENCES ARE PROCESSED" %i)
    return junction_collection



def createReference(chrSeq, chrOrder, junctionFolder,sq_header_as,sq_header_ur,sq_header_sp):
    #junctionFolder = config.get('RUN_TIME', 'out_dir')
    #junctionFolder = full_outdir
    chrFile =  os.path.join(junctionFolder, "genomeLength.txt")
    chrLengthFile = open(chrFile,"w")
    headerFile = os.path.join(junctionFolder, "sqHeader.txt")
    sqHeaderFile = open(headerFile,"w")
    refaFile = os.path.join(junctionFolder, "ref.fa")
    z = open(refaFile,'w')
    
    for c in chrOrder:
        chrLength=0
        lineCount=0
        
        seq = chrSeq[c]
        header = ">" + c
        z.write(header)
        z.write('\n')
        
        JN_Filename= os.path.join(junctionFolder,c+"_JN.fa")
        JN_File = open(JN_Filename,"r")

        chrLength+=len(seq)
        chrLengthFile.write(str(c)+"\t"+str(chrLength)+"\t"+str(lineCount)+"\n")
        sqHeaderFile.write("@SQ\tSN:"+str(c)+"\tLN:"+str(chrLength)+"\tAS:"+sq_header_as+"\tUR:"+sq_header_ur+"\tSP:"+sq_header_sp+"\n")
        
        jSeq = ''
        for line in JN_File:
            i = line.split()
            if line[:1]==">":
                continue
            
            jSeq+=i[0]

        seq+=jSeq
        x=0
        y=60
        while y < len(seq):
            z.write(seq[x:y])
            z.write('\n')
            x+=60
            y+=60
            lineCount+=1
            
        z.write(seq[x:])
        z.write('\n')
        


def main():
    global printInfo
    default_time_Format= '%Y-%m-%d %H:%M:%S'
    starting_date=time.strftime(default_time_Format)
    
    parser = _prepare_opt_parser()
    (options, args) = parser.parse_args()
    options = verify_options(options)
    

    # Getting parameter settings from cfg file
    config_file = options.c
    cfg = open(config_file,"r")
    for line in cfg:
        line=line.rstrip('\n')
        if line:
            l=line.split('=')
            if l[0]=='species':
                species = l[1]
            elif l[0]=='genome':
                genome = l[1]
            elif l[0]=='genome_fasta':
                refDefault = l[1]
            elif l[0]=='transcript_file':
                transFile = l[1]
                anno_source = os.path.splitext(os.path.basename(transFile))[0]
            elif l[0]=='out_dir':
                outdir = l[1]
            elif l[0]=='seq_length':
                seq_length = l[1]
            elif l[0]=='SQ_header_AS':
                sq_header_as = l[1]
            elif l[0]=='SQ_header_UR':
                sq_header_ur = l[1]
            elif l[0]=='SQ_header_SP':
                sq_header_sp = l[1]
            else:
                continue
        
    # Setting up the output file 
    full_outdir = outdir + "/" + species + "_" + genome + "/" + anno_source + "/" + seq_length
    if not os.path.exists(full_outdir):
        os.makedirs(full_outdir)
       
    if os.listdir(full_outdir):
        print("The desired final output directory " + full_outdir + " exists and non-empty")
        sys.exit(-1)
    
    #init_log()
    log_file = os.path.join(full_outdir,"create_Junctions.log")
    global log
    if options.d:
        log = logging.getLogger(__name__)
        log.setLevel(logging.DEBUG)
        sth = logging.StreamHandler()
        sth.setLevel(logging.DEBUG)
        log.addHandler(sth)
        fhnd = logging.FileHandler(log_file)
        fhnd.setLevel(logging.DEBUG)
        log.addHandler(fhnd)
        log.info("[logging in debug mode]")
    else:
        log = logging.getLogger(__name__)
        log.setLevel(logging.INFO)
        sth = logging.StreamHandler()
        sth.setLevel(logging.INFO)
        log.addHandler(sth)
        fhnd = logging.FileHandler(log_file)
        fhnd.setLevel(logging.INFO)
        log.addHandler(fhnd)
        log.info("[logging in info mode]")
    
    log.info("PROCESSING START: " + starting_date) 
    log.info("CREATE JUNCTIONS")
    log.info("----------------")
    log.info("Species: " + species)
    log.info("Genome: " + genome)
    log.info("Genome reference: "+refDefault)
    log.info("Transcripts file: "+transFile)
    log.info("Getting genome reference sequence for sequence reads of length:"+seq_length)
    log.info("Writing reference file to: "+full_outdir)
    
    log.info("Getting genome with no junction information...")
    chrSeq,chrOrder = getRefGenome(refDefault)
    
    log.info("Getting transcripts and exon information...")
    chrTanscripts = get_exon_information(transFile, chrSeq, full_outdir)
    
    log.info("Creating junction sequence by chromosome...")
    junctionPosCount = {}
    for chromosome in chrSeq:
        
        junctionArray = create_junctions(chromosome, chrTanscripts, chrSeq, int(seq_length))
    
        log.debug("%s junctionArray %d" %(chromosome, len(junctionArray)))
        genomeSeq = chrSeq[chromosome]
        pos_count=1
        junctionPosCount[chromosome]=pos_count
        
        log.debug("Remove duplicated junction sequences...")
        ###jn_fa = os.path.join(full_outdir,chromosome+"_JN.header")
        ###junction_collection = modify_junction_sequences(seq_length, jn_fa)
        junction_collection = modify_junction_sequences(seq_length,junctionArray)
        log.debug(len(junction_collection))
        # print out new junction fa
        newfastaFilename = os.path.join(full_outdir,chromosome+"_JN.fa")
        newjunctionsFasta = open(newfastaFilename,'a')
        newjunctionSeqForReferenceFilename = os.path.join(full_outdir,chromosome+"_JN.txt")
        newjunctionSeqForReference = open(newjunctionSeqForReferenceFilename,'a')
        """
        junStartSorted=[]
        for start in junction_collection:
            junStartSorted.append(int(start))
            junStartSorted.sort()
        
        for start in junStartSorted:
            for junction_object in junction_collection[str(start)]:
        """
        for junction_object in junction_collection:
            m=0
            seqFromGenome = ''
            strand = junction_object.strand
            coordSorted = junction_object.coords
            if strand=="-1":
                coordSorted.reverse()
            log.debug(coordSorted)
            while m < len(coordSorted):
                log.debug("m=%d" %m)
                seqFromGenome += genomeSeq[int(coordSorted[m])-1:int(coordSorted[m+1])]
                m+=2
            seqpad = seqFromGenome+pad+'\n'
            fastaSeqLength = len(seqFromGenome+pad)
            currentFastaRefLength = junctionPosCount[chromosome]
            junctionPosCount[chromosome]+=fastaSeqLength
            countInfo = str(currentFastaRefLength)+"-"+str(currentFastaRefLength+fastaSeqLength-1)
                
            coordList = ','.join(coordSorted)
            header = ">"+ coordList +"|"+countInfo+"|"+strand
            newjunctionsFasta.write(header)
            newjunctionsFasta.write("\n")

            newjunctionsFasta.write(seqpad)
            newjunctionSeqForReference.write(seqpad)


    log.info("Creating final genome plus junction reference...")
    createReference(chrSeq, chrOrder, full_outdir,sq_header_as,sq_header_ur,sq_header_sp)
    
    end_date=time.strftime(default_time_Format)
    log.info("PROCESSING SUCCESSFULLY COMPLETED: " + end_date)
    
#def init_log ():
 #   console = log.StreamHandler(sys.stdout)
 #   console.setFormatter(log.Formatter('%(levelname)s %(message)s'))
 #   log.getLogger('')
 #   log.getLogger('').addHandler(console)
 #   log.getLogger('').setLevel(log.NOTSET)
 
# prepare options
def _prepare_opt_parser():
    
    parser = OptionParser(__doc__)
    parser.add_option("-c",dest="c",help="config file",metavar="c")
    parser.add_option("-d",dest="d",action="store_true",help="debug mode",metavar="d")

    return parser

# verify that all necessary parameters are set
def verify_options(options):
    
    if not options.c:
        log.critical("Invalid arguments. Missing argument -c.")
        sys.exit(-1)
        
    return options 
    
if __name__ == '__main__':
    main()
    
