#!/usr/bin/env python
#coding:utf-8
# Author:   Yaron Butterfield
# Purpose:  Filter SAM files with multiple alignments per read pair to include only one paired 
#           end set of reads suitable for downstream analysis such as with Picard and JAGuaR
# Created: 12/20/2012

import sys
import samIterator, samFlags, convertCigar

    
	     
    
def selectReads(fileName):
    """
     Go through SAM files and take first valid paired end read set.  The assumption 
     is that this can be determined by the flag and for each end, take the read with the largest 
     length of read that aligned
     
     If flag is < 129, 'the read is first read in a pair'
     if flag is >= 129, 'the read is the second read in a pair'
     
     See JAGuaR's samFlag's for details.
     
    """
    f = open(fileName,'r')
    
    linePair=0
    query=0
    mate=0
    currentQueryName=''
    z1=samIterator.align()
    z2=samIterator.align()
    
    lineCount=0
    
    readBucket=[]
    binLimit=0
    
    
    for line in f:
    
	if line[0]=="@":
	    print line[:-1]
	    continue    

	x = line.split()
	
	z=samIterator.align() #create an empty object
	samIterator.readSAM(z,x,1,1) #z1 will be modified and will contain the contents of x when this function returns
	
	if z.qname <> currentQueryName:
	    # indication of a new read
	    linePair=0
	    query=0
	    mate=0	
	    z1=samIterator.align()
	    z2=samIterator.align()
	    currentQueryName = z.qname
	
	if z.flag > 181:
	    continue
	
	if z.flag < 129:   # the read is the first in a pair
	    linePair+=1	    
	    if query==1:
		# query seen before, take one with longest read aligned
		if len(z.seq) > len(z1.seq):
		    z1 = z
	    else:
		z1 = z
	    query=1
	   	    
	if z.flag >= 129:  # the read is the second in a pair
	    linePair+=1
	    if mate==1:
		# mate seen before, take one with longest read aligned
		if len(z.seq) > len(z2.seq):
		    z2 = z
	    else:
		z2 = z
	    mate=1
		
		
	if z1.qname <> '' and z2.qname <> '' and z1.qname == z2.qname:
	    z1.mrnm = z2.rname
	    z1.mpos = z2.pos
	    z2.mrnm = z1.rname
	    z2.mpos = z1.pos	    
	    print samIterator.createline(z1)
	    print samIterator.createline(z2)   
	    z1=samIterator.align()
	    z2=samIterator.align()	    
	    


	 
	 

    
if __name__=='__main__':
    
    fileName = sys.argv[1]
    
    selectReads(fileName)
    
