#!/usr/bin/env python
#coding:utf-8
# Author:  Yaron S. Butterfield
# Purpose: Read a line from a SAM file for downstram analysis
# Created: 07/13/2009

import sys
from optparse import OptionParser
import re

class align(object):  
    def __init__(self):
        self.__class__.__name__
        self.qname = '' # [^ \t\n\r]+ Query pair NAME if paired; or Query NAME if unpaired
        self.flag = 0 #  [0-9]+ [0,216-1] bitwise FLAG 
        self.rname = '' # [^ \t\n\r@=]+ Reference sequence NAME (often a chromosome name)
        self.pos = 0 # [0-9]+ [0,229-1] 1-based leftmost POSition/coordinate of the clipped sequence
        self.mapq = 0 # [0-9]+ [0,28-1] MAPping Quality (phred-scaled posterior probability that the mapping position of this read is incorrect) 
        self.cigar = '' # ([0-9]+[MIDNSHP])+|\* extended CIGAR string
        self.mrnm = ''  # MRNM Reference that matching PET (Mate) aligns to
        self.mpos = 0 # MPOS [0-9]+ [0,229-1] 1-based leftmost Mate POSition of the clipped sequence
        self.isize = 0 # ISIZE -?[0-9]+ [-229,229] inferred Insert SIZE - calculate only if both Mate pairs align to the same reference
        self.seq = '' 
        self.qual = '' # QUAL [!-~]+|\* query QUALity; ASCII-33 gives the Phred base quality 
        #self.tag = '' # [A-Z][A-Z0-9] TAG
        #self.vtype = '' # VTYPE [AifZH] Value TYPE
        #self.value = '' # VALUE [^\t\n\r]+ match <VTYPE> (space allowed)
        self.tags = []
        
        # RNA-seq specific
        self.tempCigar = ''
        self.junction = 0
        self.splice_size = 0
        self.softclip = 0
        self.softclipEnd =0
        self.junctionCoord ='' # if there is an alignment to a junction region, store the position in the exon-exon junction
        self.distanceFromJunctionEnd = 0
        self.genomeAligned = 0
        self.readAlignedToPad = 0
        self.readAlignedToPadStart = 0
        self.setUnmapped = 0
        self.dropped_indels_num = 0 #the number of indel event(s) dropped by softclipping
        self.dropped_indels_bases = 0 #the number of bases in the indel event(s) dropped by softclipping
        self.intron_bool = []
        self.add_to_pos = 0
        
def createline(z):
    line = str(z.qname)+'\t'
    line += str(z.flag)+'\t'
    line += str(z.rname)+'\t'
    line += str(z.pos)+'\t'
    line += str(z.mapq)+'\t'
    line += str(z.cigar)+'\t'
    line += str(z.mrnm)+'\t'  
    line += str(z.mpos)+'\t'
    line += str(z.isize)+'\t'
    line += str(z.seq)+'\t'
    line += str(z.qual)+'\t'
    tags = z.tags
    tagline=''
    if len(tags)>0:
        for t in tags:
            tagline += t+'\t'
    line+=tagline
    return line.strip()    
    

def readSAM(z, x, mt, chrom):
    """
    z is an align() object
    x is an array of strings created by running split() on a line from a sam file
    mt is a boolean from the parameter --do_not_check_mito_naming 

    """
    
    #we're about to re-use this object so reinitialize everything we are not about to set
    z.tempCigar = ''
    z.junction = 0
    z.splice_size = 0
    z.softclip = 0
    z.softclipEnd =0
    z.junctionCoord ='' # if there is an alignment to a junction region, store the position in the exon-exon junction
    z.distanceFromJunctionEnd = 0
    z.genomeAligned = 0
    z.readAlignedToPad = 0
    z.readAlignedToPadStart = 0
    z.setUnmapped = 0
    z.dropped_indels_num = 0 
    z.dropped_indels_bases = 0
    z.intron_bool = []  
    z.add_to_pos = 0
    
    z.qname=x[0]
        
    z.flag=int(x[1]) 
    
    z.rname=x[2]
    
    if chrom:
        pass
    else:
        if 'chr' in x[2]:
            z.rname = x[2][3:]
        
    if mt:
        pass
    else:
        if z.rname[-1] =='M':
            z.rname += 'T'
            
        
    z.pos= int(x[3])
    z.mapq= int(x[4])
    z.cigar= x[5]
    z.mrnm = x[6]
    z.mpos = int(x[7])
    z.isize = int(x[8])
    z.seq = x[9] 
    
    z.qual = x[10] 
   
    tagsLength=len(x)-11 #sam records have 11 required fields, additional tags are optional
    if len(x)>11:
        z.tags=x[11:]
    else:
        z.tags=[]

    return

def getTagDetails(z):
    tagDict={}
    for t in z.tags:
        i = t.split(':')
        tag = i[0]
        if i[1]=='i':
            tagDict[tag]=int(i[2])
        else:
            tagDict[tag]=i[2]
    return tagDict

if __name__=='__main__':

    parser = OptionParser(description=__doc__)
    parser.add_option("-s",dest="s",help="sam file",metavar="s")
        
    (options, args)=parser.parse_args()
        
    samFile = options.s
    
    f = open(samFile,'r')
    
    for line in f:
        x = line.split()
        z = readSAM(x)
        
        rname = z.rname
        
def editTag(z, tag_name, new_tag_value):
    n = 0
    for tag in z.tags:
        if re.match(tag_name, tag):
            tag_prefix = re.match("(.*:.*:)", tag).group(1)
            tag = tag_prefix+new_tag_value
            z.tags[n] = tag
            break
        n = n+1
    return z        